#!perl -w
use strict;
use warnings;
use utf8;
use constant MECABRC => './mecabrc';
use Encode;
use Text::MeCab;
use Lingua::JA::Regular::Unicode qw/katakana2hiragana/;

if(@ARGV != 1){
    print <<USAGE;
USAGE: ./add_rubytag.pl INPUT_TEXT
USAGE
    exit -1;   
}

run(@ARGV);

exit 0;

#####################################################################

sub run {
    my $input = shift;

    my $parser = Text::MeCab->new(
        rcfile=> MECABRC,
    );

    my $encoding = Encode::find_encoding('utf8');

    my $result = q{};
    for my $text (split /(\s+)/, $input) { 
        foreach (
            my $node = $parser->parse($text);
            $node;
            $node = $node->next
        ) {
            next if $node->stat =~ /[23]/; # skip MECAB_(BOS|EOS)_NODE

            my $surface = $encoding->decode($node->surface);
            my $feature = $encoding->decode($node->feature);
            my $read = (split /,/, $feature)[7];
            
            if($read ne q{} && $surface ne $read) {
                $result .= add_ruby_tag($surface, $read);
            }else {
                $result .= $surface;
            }
        }
    }
    
    print $encoding->encode($result), "\n";
}

sub add_ruby_tag {
    my ($surface, $read) = @_;
    my $read_hiragana = katakana2hiragana($read);
    if($surface ne $read_hiragana){
        return "<ruby>" . $surface.  "<rt>" . $read_hiragana . "</ruby>";
    } else {
        return $surface;
    }
}

